@@ -1,22 +1,56 @@
 --
--- Copyright (c) 2021-2022 Zeping Lee
+-- Copyright (c) 2021-2024 Zeping Lee
 -- Released under the MIT license.
 -- Repository: https://github.com/zepinglee/citeproc-lua
 --
 
 local style_module = {}
 
-local dom = require("luaxml-domobject")
+local dom
+local element
+local ir_node
+local output
+local node_names
+local util
 
-local Element = require("citeproc-element").Element
-local IrNode = require("citeproc-ir-node").IrNode
-local Rendered = require("citeproc-ir-node").Rendered
-local SeqIr = require("citeproc-ir-node").SeqIr
-local PlainText = require("citeproc-output").PlainText
-local DisamStringFormat = require("citeproc-output").DisamStringFormat
-local util = require("citeproc-util")
+if kpse then
+  dom = require("luaxml-domobject")
+  element = require("citeproc-element")
+  ir_node = require("citeproc-ir-node")
+  output = require("citeproc-output")
+  node_names = require("citeproc-node-names")
+  util = require("citeproc-util")
+else
+  dom = require("citeproc.luaxml.domobject")
+  element = require("citeproc.element")
+  ir_node = require("citeproc.ir-node")
+  output = require("citeproc.output")
+  node_names = require("citeproc.node-names")
+  util = require("citeproc.util")
+end
+
+local Element = element.Element
+local IrNode = ir_node.IrNode
+local Rendered = ir_node.Rendered
+local SeqIr = ir_node.SeqIr
+local PlainText = output.PlainText
+local DisamStringFormat = output.DisamStringFormat
 
 
+---@class Style: Element
+---@field class string
+---@field default_locale string?
+---@field version string?
+---@field initialize_with_hyphen boolean
+---@field page_range_format string?
+---@field demote_non_dropping_particle boolean?
+---@field info Element
+---@field locales { [string]: Locale }
+---@field macros { [string]: Element }
+---@field citation Citation
+---@field intext Citation?
+---@field bibliography Bibliography?
+---@field has_disambiguate boolean
 local Style = Element:derive("style")
 
 function Style:new()
@@ -46,12 +80,14 @@ function Style:parse(xml_str)
     return nil
   end
   local style_node = csl_xml:get_path("style")[1]
-  if not csl_xml then
+  if not style_node then
     error('Element "style" not found.')
   end
   return Style:from_node(style_node)
 end
 
+---@param node any
+---@return Style
 function Style:from_node(node)
   local o = Style:new()
 
@@ -67,7 +103,7 @@ function Style:from_node(node)
 
   -- Inheritable Name Options
   -- https://docs.citationstyles.org/en/stable/specification.html#inheritable-name-options
-  o.name_inheritance = require("citeproc-node-names").Name:new()
+  o.name_inheritance = node_names.Name:new()
   Element.make_name_inheritance(o.name_inheritance, node)
 
   if o.page_range_format == "chicago" then
@@ -102,6 +138,11 @@ function Style:from_node(node)
     end
   end
 
+  o.has_disambiguate = false
+  if #node:query_selector('[disambiguate="true"]') > 0 then
+    o.has_disambiguate = true
+  end
+
   return o
 end
 
@@ -112,66 +153,63 @@ Style._default_options = {
   ["demote-non-dropping-particle"] = "display-and-sort",
 }
 
-function Style:set_lang(lang, force_lang)
-  local default_locale = self:get_attribute("default-locale")
-  if lang then
-    if default_locale and not force_lang then
-      self.lang = default_locale
-    end
-  else
-    self.lang = default_locale or "en-US"
-  end
-end
-
 
+---@class Info: Element
+---@field author table?
+---@field contributors table[]
+---@field citation_format string?
+---@field fields string[]
+---@field id string?
+---@field issn string?
+---@field eissn string?
+---@field issnl string?
+---@field links string[]
+---@field link_rel table<string, string>
+---@field independent_parent string?
+---@field published string?
+---@field rghts string?
+---@field summary string?
+---@field title string?
+---@field title_short string?
+---@field updated string?
 local Info = Element:derive("info")
 
 
 function Info:from_node(node)
+  ---@type Info
   local o = Info:new()
 
-  -- o.authors = nil
-  -- o.contributors = nil
-  o.categories = {}
-  o.id = nil
-  -- o.issn = nil
-  -- o.eissn = nil
-  -- o.issnl = nil
-  o.links = {
-    independent_parent = nil,
-  }
-  -- o.published = nil
-  -- o.rights = nil
-  -- o.summary = nil
-  o.title = nil
-  -- o.title_short = nil
-  o.updated = nil
+  o.contributors = {}
+  o.fields = {}
+  o.links = {}
+  o.link_rel = {}
 
   for _, child in ipairs(node:get_children()) do
     if child:is_element() then
       local element_name = child:get_element_name()
-      if element_name == "category" then
+
+      if element_name == "author" or element_name == "contributor" then
+        -- TODO
+      elseif element_name == "category" then
         local citation_format = child:get_attribute("citation-format")
+        local field = child:get_attribute("field")
         if citation_format then
-          o.categories.citation_format = citation_format
+          o.citation_format = citation_format
+        elseif field then
+          table.insert(o.fields, field)
         end
-
-      elseif element_name == "id" then
-        o.id = child:get_text()
-
       elseif element_name == "link" then
         local href = child:get_attribute("href")
         local rel = child:get_attribute("rel")
-        if href and rel == "independent-parent" then
-          o.links.independent_parent = href
+        if href and rel then
+          table.insert(o.links, href)
+          o.link_rel[href] = rel
+          if rel == "independent-parent" then
+            o.independent_parent = href
+          end
         end
-
-      elseif element_name == "title" then
-        o.title = child:get_text()
-
-      elseif element_name == "updated" then
-        o.updated = child:get_text()
-
+      else
+        o[element_name] = child:get_text()
       end
     end
   end
